// 	$Id: geneticCodeHolder.cpp 920 2006-09-21 09:26:12Z ninio $	


#include "geneticCodeHolder.h"

const geneticCodeString geneticCodeHolder::nuclearStandard(
#include "replacementMatrixSource/nuclearStandard.code"
);

const geneticCodeString geneticCodeHolder::nuclearEuplotid(
#include "replacementMatrixSource/nuclearEuplotid.code"
);

const geneticCodeString geneticCodeHolder::nuclearCiliate(
#include "replacementMatrixSource/nuclearCiliate.code"
);

const geneticCodeString geneticCodeHolder::nuclearBlepharisma(
#include "replacementMatrixSource/nuclearBlepharisma.code"
);

const geneticCodeString geneticCodeHolder::mitochondriaYeast(
#include "replacementMatrixSource/mitochondriaYeast.code"
);

const geneticCodeString geneticCodeHolder::mitochondriaVertebrate(
#include "replacementMatrixSource/mitochondriaVertebrate.code"
);

const geneticCodeString geneticCodeHolder::mitochondriaProtozoan(
#include "replacementMatrixSource/mitochondriaProtozoan.code"
);

const geneticCodeString geneticCodeHolder::mitochondriaInvertebrate(
#include "replacementMatrixSource/mitochondriaInvertebrate.code"
);

const geneticCodeString geneticCodeHolder::mitochondriaFlatworm(
#include "replacementMatrixSource/mitochondriaFlatworm.code"
);

const geneticCodeString geneticCodeHolder::mitochondriaEchinoderm(
#include "replacementMatrixSource/mitochondriaEchinoderm.code"
);

const geneticCodeString geneticCodeHolder::mitochondriaAscidian(
#include "replacementMatrixSource/mitochondriaAscidian.code"
);

